package appletlaser5;

import classement.EnsembleTrie;
import classement.Parcours;
import espace3D.Vecteur3D;
import fonctions.Constantes;
import photons.*;
import atomes.*;
import simu.*;


/**
 * Algorithme de simulation pour SimuLab.
 * Refroidissement atomique par laser 
 */
public class Laser5 extends SimuApplet {
    //---------------------------
    // variables globales de la classe Laser5
    //---------------------------
    Dispositif_Laser laser;
    int nbatomes; // nombre d'atomes
    double tau; //temps moyen de desexcitation des atomes
    Vecteur3D dirinit; //Direction initiale du faisceau d'atomes
    double dureesim; //dure de la simulation en nanosecondes
    //---------------------------
    double tempsecoule;//date de fin du dernier cycle d'actions
    double tempspreced;//date de fin de l'avant dernier cycle d'actions
    //---------------------------
    EnsembleTrie echeancier;// chancier des changements d'tat des atomes. 
    EnsembleTrie population; //population des atomes, tris par vitesses initiales croissantes
    //----------------
    int nbcycles;
    //nombre de cycles d'actions avant affichage de la distribution des vitesses individuelles 
    int []compteurs;
    //compteurs d'actions de diffrents types observes pendant un cycle
    //----------------
    // constructeur d'un objet de la classe
    public Laser5() {
        appletDoc("laser5_doc.xml");
    }
    public void initCalculLive(ListeValeurs in) throws SimuException {
        //------------------------------------- 
        // rcupration des paramtres d'entre
        //------------------------------------
        // sur les atomes : 
        double masse = in.lireDouble("masseatome");
        double lambdatome=in.lireDouble("lambdatome");
        tau=in.lireDouble("tauatome");
        double g = in.lireDouble("g1surg0");
        double vitesseinitiale=in.lireDouble("vitessemoy");
        double modulationvitesse = in.lireDouble("modulationvitesse");
        //--
        //les atomes sont envoys dans une direction diagonale
        double x = 1.0/Math.sqrt(3.0);
        double y = 1.0/Math.sqrt(3.0);
        double z = 1.0/Math.sqrt(3.0);
        dirinit = new Vecteur3D(x,y,z);
        //--------------------------
        //sur le dispositif laser :
        //dtermination de la longueur d'onde du laser
        double vsurc = vitesseinitiale*x/(Constantes.c);
        double doppler = 1.0 + vsurc;
        double eta = in.lireDouble("frequencelaser");
        double reglagelaser = 1+eta*vsurc;
        double lambdalaser = lambdatome*doppler/reglagelaser;
        //une valeur 0 du rglage laser implique que la frquence du laser
        //vue par un atome avec sa vitesse et sa direction initiale
        //sera exactement sa frquence de rsonnance
        //(efficacit maximale)
        //---------------------------
        // dtermination de l'intensit du laser
        double cp = Constantes.c*1.0E-8;
        double hp = Constantes.h*1.0E34;
        double f = Constantes.c/(lambdatome*1.0E6);
        double numerateur = 2*cp*cp*1.0E2;
        double denominateur = hp*f*f*f;
        double tauxRenc =numerateur/denominateur;
        double reglageint = in.lireDouble("intensitelaser");
        double intensitelaser = (1/(3*tau))*(1/tauxRenc)*reglageint*1.0E9;
        //une valeur 1 du reglage assure l'galit entre les nombres de dsexcitations spontanes et induites
        //--------------------------------------
        double sensibilite = (1/tau)*(1/(2*Math.PI));
        double largeurlaser = in.lireDouble("largeurlaser");
        //--------------------------------------
        //sur la marche de la simulation :
        nbatomes = (int) in.lireDouble("nbatomes");
        dureesim = (int) in.lireDouble("dureesim");
        //----------------------------
        //initialisations
        //----------------------------
        tempsecoule = 0;
        tempspreced = 0;
        echeancier = new EnsembleTrie();
        population = new EnsembleTrie();
        nbcycles = 0;
        compteurs = new int[4];
        //----------------------------
        //initialisation du dispositif laser
        laser = new Dispositif_Laser(lambdalaser, intensitelaser, largeurlaser, sensibilite);
        //----------------------------
        //initialisation des atomes
        //(tous mis  l'tat dsexcit)
        //de l'chancier et de la population
        for(int i=0;i<nbatomes;i++){
            Atome atome = new Atome(masse,vitesseinitiale,modulationvitesse,dirinit,lambdatome,tau,g,laser);
            echeancier.insertion(atome.prochaineexcitation(laser));
            population.insertion(atome);
        }
       //-----------------------
       //dimensionnement fentres
       // Suivi des vitesses individuelles
       // fentre des vitesses par rapport au temps
       Affichage f0=getAffichage("svitesses");
       Affichage []f00=f0.sousAff;
       Affichage f01=f00[0];
       Affichage f02=f00[1];
       f01.xdebut="0";
       f01.xfin=""+dureesim;
       f01.ydebut="0";
       f01.yfin=""+(1.5*vitesseinitiale);
       //fentre de la distribution des vitesses
       f02.xdebut="0";
       f02.xfin="3.0";
       f02.ydebut="0";
       f02.yfin=""+nbatomes;
       //------------------------
    // Suivi des vitesses moyennes
       // fentre de la vitesse moyenne
       Affichage f3=getAffichage("smoyenne");
       Affichage []f30=f3.sousAff;
       Affichage f31=f30[0];
       Affichage f32=f30[1];
       f31.xdebut="0";
       f31.xfin=""+dureesim;
       f31.ydebut="0";
       f31.yfin=""+(1.5*vitesseinitiale);
       //fentre de l'orientation
       f32.xdebut="0";
       f32.xfin=""+dureesim;
       f32.ydebut="0";
       f32.yfin="180";
       //------------------------
       // Suivi des excitations
       // fentre des excitations
       Affichage f1=getAffichage("sexcitations");
       Affichage []f10=f1.sousAff;
       Affichage f11=f10[0];
       Affichage f12=f10[1];
       f11.xdebut="0";
       f11.xfin=""+dureesim;
       f11.ydebut="0";
       f11.yfin=""+(10);
       //fentre des excits
       f12.xdebut="0";
       f12.xfin=""+dureesim;
       f12.ydebut="0";
       f12.yfin="1.0";
       //------------------
      // Suivi des parcours
       // fentre de frquence moyenne des changements d'orientation
       Affichage f2=getAffichage("schangement");
       Affichage []f20=f2.sousAff;
       Affichage f21=f20[0];
       Affichage f22=f20[1];
       f21.xdebut="0";
       f21.xfin=""+dureesim;
       f21.ydebut="0";
       f21.yfin=""+(1.0);
       //fentre du libre parcours moyen
       f22.xdebut="0";
       f22.xfin=""+dureesim;
       f22.ydebut="0";
       f22.yfin=""+2*vitesseinitiale*1;
    }     
    public ListeValeurs calculLive() throws SimuException {
    	Action action = null;
    	Atome atome = null;
        ListeValeurs out = new ListeValeurs();
        //--------------------------------------------------------------------------
        if(tempsecoule > dureesim) return null;
        for(int i=0;i<3;i++)compteurs[i] = 0;
        //---------------------------
        for(int i=0; i<2*nbatomes;i++){
            //extraction de la prochaine action 
            //inscrite dans l'chancier
            action = (Action) echeancier.prochaine();
            //----------------------------
            //excution de cette action
            //incluant le calcul de la prochaine action du mme processus
            //et son inscription dans l'chancier
            echeancier.insertion(action.execute());
            //incrmentation du compteur adquat
            action.incremente(compteurs);
        }
        //---------------------------------------------------------------------------
        //un cycle d'actions vient de s'achever (nbatomes actions)
        //TRAITEMENTS DE FIN DE CYCLE
        //---------------------------------------------
        //incrmentation du nombre de cycles effectus;
        nbcycles = nbcycles +1;
        //--------------------------------------------
        //extraction du temps coul et de la vitesse du dernier atome modifi
        atome=action.get_atome();
        tempsecoule = atome.get_date();
        Vecteur3D vitesse = atome.get_vitesse();
        //--------------------------------------------
        //tri des atomes selon les vitesses croissantes
        // et calculs de moyennes
        compteurs[3]=0;
        double somme_parcours = 0;
        Vecteur3D somme_vitesses = new Vecteur3D(0,0,0);
        double somme_duree = 0;
        //--------------------
        EnsembleTrie tri_vitesses = new EnsembleTrie();
        Parcours parcours = new Parcours(population);
        for (int i=0;i<nbatomes;i++){
        	if(parcours.hasNext()){
        		Atome a = (Atome) parcours.next().get_element();
        		tri_vitesses.insertion(a);
        		compteurs[3]=compteurs[3]+a.get_etat();
        		somme_parcours=somme_parcours +a.get_parcours();
        		Vecteur3D v = a.get_vitesse();
        		somme_vitesses.lineWith(1, 1, v);
        		somme_duree = somme_duree+a.get_delai();
        	}
        }
        double lpm = (somme_parcours/nbatomes);
        somme_vitesses.line(1.0/nbatomes);
        double dm = (somme_duree/nbatomes);
        double freqmoy = 1.0/dm;
        //----------------------------------------------
        //AFFICHAGES
        //-----------------------------------------------
        //affichage de la distribution cumule des vitesses
        //on affiche celle i compltement 
        //qu'aprs avoir ffectu un nombre de cycles 
        //qui soit un mutiple entier du nombre d'atomes
        //sinon on n'affiche que la valeur mdiane
        //extraction de la vitesse mdiane
        double vitesse_med = tri_vitesses.valeur_med();
        int rang_med =(int) Math.ceil(nbatomes/2);
        //---
        if(Math.IEEEremainder(nbcycles-1, 10*nbatomes)==0){
        	Parcours p = new Parcours(tri_vitesses);
            for (int i=0;i<nbatomes;i++){
        	    if(p.hasNext()){
        		    Atome a = (Atome) p.next().get_element();
        	        out.ajouterDouble("vitesse",a.get_vitesse().norme()/vitesse_med);
				    out.ajouterDouble("rang", i*1.0);
        	    }
            }
        }
        out.ajouterDouble("vitesse", 1.0);
        out.ajouterDouble("rang",rang_med);
        //---
        double tempsnormalise=Math.round(tempsecoule);
        //affichage des vitesses individuelles
        out.ajouterDouble("temps",tempsecoule);       
        out.ajouterDouble("vitesseat",vitesse.norme());
        //-----------------------------
        //affichage de la norme du vecteur vitesse moyen
        out.ajouterDouble("vitvecmoy", somme_vitesses.norme());
        //-----------------------------
        //affichage de l'angle avec la direction initiale
        double cos = somme_vitesses.scalWith(dirinit)/somme_vitesses.norme();
        double angle = Math.acos(cos)*(180/Math.PI);
        out.ajouterDouble("vitprjmoy", angle);
        //------------------------------
        //affichage du nombre net d'excitations par atomes
        double excitationsnettes = compteurs[0];//-compteurs[1];
        double nbunitestemps = (tempsecoule-tempspreced)/(10*tau);
        double nbparatomes = excitationsnettes/(nbatomes*nbunitestemps);
        out.ajouterDouble("nbexcitations",nbparatomes);
        //-----------------------------
       //affichage de la proportion d'atomes excits
        double proportion = (compteurs[3]*1.0)/nbatomes;
        out.ajouterDouble("nbexcites", proportion);
        //-----------------------------
        //affichage de la frquence moyenne des changements d'orientations
        out.ajouterDouble("freqmoy", freqmoy);
        //-----------------------------
        //affichage du libre parcours moyen
        out.ajouterDouble("lpm",lpm);
        //------------------------------
        out.ajouterString("tempsecoule","temps coul en nano secondes : "+tempsnormalise);
        //------------------------------
        tempspreced = tempsecoule;
        return out;
    }
}