package atomes;

import classement.Element;
import espace3D.Vecteur3D;
import fonctions.Tirages;
import fonctions.Constantes;
import fonctions.Lorentz;
import photons.*;

/**
 * classe d'objets reprsentant des atomes
 * la classe est conue de manire
 *  accepter simplement d'autres actions que
 * les actions actuelles (excitations et desexcitations)
 */
public class Atome extends Element{
    private int etat;
    //peut prendre deux valeurs 
    //0 : dsexcit //1 : excit
    private double masse; //masse de l'atome (masse atomique)
    private Vecteur3D vitesse; // vitesse de l'atome (en m/s)
    private double freqres; // frquence de rsonnance // (en GHZ)
    private double tau; //dure de vie moyenne du niveau excit (nanosecondes)
	private double g; //rapport poids statistiques (sup/inf)
    //-----------------
    private double date;//date du dernier changement d'tat
    private double delai; //delai jusqu' la prochaine action
    //-----------------
    private double libre_parcours;//longueur du parcours entre deux changements
    // d'orientations
    // Attributs de raction  un photon
    private Lorentz psi;
    private double tauxRenc;
    //------------------
    public int get_etat(){return etat;}
    public Vecteur3D get_vitesse(){return vitesse;}
    public double get_tau(){return tau;}
    public double get_freqres(){return freqres;}
    public double get_date(){return date;}
    public double get_delai(){return delai;}
    public double get_parcours(){return libre_parcours;}
    //-----------------
    private Desexspt dexc_spt;
    private Desexind dexc_ind;
    private Exitation excitation;
    /**
     * Constructeur d'un atome
     * @param masse (en nombre de masse atomique)
     * @param vitesseinitiale (en m/s)
     * @param lambdatome //longueur d'onde de rsonnance (en nm)
     * @param tau // temps moyen de desxcitation spontane (en ns )
     * return
    */
    public Atome(double masse, double vitesse_initiale, double modulationvitesse, Vecteur3D dirinit, double lambdatome, double tau, double g, Dispositif_Laser laser) {
    	super(vitesse_initiale);
        //------------------
        this.masse = masse;
        this.vitesse = dirinit.copy();
        double vitesseff = Tirages.tirage_gaussien(vitesse_initiale,modulationvitesse,"fwhm");
        (this.vitesse).line(vitesseff);
        valeur = vitesse.norme();
        this.freqres = Constantes.c/lambdatome;
        this.tau = tau;
		this.g = g;
        //------------------
        this.etat = 0;
        this.date = 0;
        this.delai = 0;
        //-----------------
      //initialisation d'actions prototypes
        //pour viter de les crer  chaque fois
        this.dexc_spt = new Desexspt(this,laser);
        this.dexc_ind = new Desexind(this,laser);
        this.excitation = new Exitation(this,laser);
    //initialisation de psi et de tauxRenc
        double gammasur4pi=(1/tau)*1/(4*Math.PI);
        this.psi = new Lorentz(freqres,gammasur4pi);
        double cp = Constantes.c*1.0E-8;
        double hp = Constantes.h*1.0E34;
        double f = freqres/1.0E6;
        double numerateur = 2*cp*cp*1.0E2;
        double denominateur = hp*f*f*f;
        this.tauxRenc =numerateur/denominateur;
    }
    /**
     * calcul de la prochaine dsexcitation concernant cet atome
     * @param laser : objet contenant les donnes du dispositif laser
     * @param chancier dans lequel sera enregistr cette action
     * @return
     */
    //public void prochainedesexcitation(Dispositif_Laser laser, EnsembleTrie echeancier){
    public Action prochainedesexcitation(Dispositif_Laser laser){
        double delai_dexc_spt = Tirages.tirage_exponentiel(tau);
        double freq_dexc_spt = Tirages.tirage_lorentzien(freqres, tau);//frquence du photon mis
        Vecteur3D dir_spt = new Vecteur3D(1.0);//direction du photon mis
        dexc_spt.set(date+delai_dexc_spt, freq_dexc_spt, dir_spt);
        //---------
        Photon inducteur = laser.getphoton(psi, vitesse, tauxRenc);
        //rcupration des donnes du photon 
        double delai_dexc_ind = inducteur.get_delai();
        double freq_dexc_ind = inducteur.get_frequence();//frquence du photon inducteur
        Vecteur3D dir_dexc_ind = inducteur.get_dir();//direction du photon inducteur
        dexc_ind.set(date+delai_dexc_ind, freq_dexc_ind, dir_dexc_ind);
		//---------
		Action actionenretour;
        //inscription dans l'chancier de la desexcitation la plus proche dans le temps
        if(dexc_spt.get_valeur()<=dexc_ind.get_valeur()){
            //echeancier.insertion(dexc_spt);
            delai = delai_dexc_spt;
			actionenretour = dexc_spt;
        }else{
        	//echeancier.insertion(dexc_ind);
        	delai = delai_dexc_ind;
			actionenretour = dexc_ind;
        } 
        libre_parcours = vitesse.norme()*delai;
		return actionenretour;
    }
    /**
     * calcul de la prochaine excitation concernant cet atome
     * @param laser : objet contenant les donnes du dispositif laser
     * @param chancier dans lequel sera enregistr cette action
     * @return
     */
    public Action prochaineexcitation(Dispositif_Laser laser){
    	Photon excitateur = laser.getphoton(psi, vitesse, g*tauxRenc);
        //rcupration des donnes du photon
        double delai_exc = excitateur.get_delai();
        double freq_exc = excitateur.get_frequence();//frq. du photon excitateur
        Vecteur3D dir_exc = excitateur.get_dir();//direction du photon excitateur
        excitation.set(date+delai_exc, freq_exc, dir_exc);
        //---------
        //inscription dans l'chancier
        //echeancier.insertion(excitation);
        delai = delai_exc;
        libre_parcours = vitesse.norme()*delai;
		return excitation;
    }
	//---------------------------------------------------------------------
    public void excite(double newdate, double frequence, Vecteur3D direction,int newstate){ 
        etat = newstate;
        date = newdate;
        double dv = getdv(frequence);
        vitesse.lineWith(1, dv, direction);//modification de la vitesse
		//actualisation du critre de tri
		valeur = vitesse.norme();
    }
    public void desexspt(double newdate, double frequence, Vecteur3D direction,int newstate){ 
        etat = newstate;
        date = newdate;
        double dv = getdv(frequence);
        vitesse.lineWith(1, dv, direction);//modification de la vitesse
		//actualisation du critre de tri
		valeur = vitesse.norme();
    }
    public void desexind(double newdate, double frequence, Vecteur3D direction,int newstate){ 
        etat = newstate;
        date = newdate;
        double dv = getdv(frequence);
        vitesse.lineWith(1, dv, direction);//modification de la vitesse
		//actualisation du critre de tri
		valeur = vitesse.norme();
    }
    private double getdv(double freq){
        double hp= Constantes.h*1.0E34;
        double cp= Constantes.c*1.0E-8;
        double mp= Constantes.m*1.0E27;
        return (hp*(1.0E-6)*freq)/(masse*mp*cp);
    }
}
