package espace3D;

import java.util.Random;

/**
 * classe d'objets donnant la reprsentation cartsienne
 * d'un vecteur dans un espace 3D
 */
public class Vecteur3D {
	//pour effectuer des tirages alatoires
	public static Random tirage = new Random();
	//--------------------
	protected double x,y,z;
	//-----------------------
	     /**
	     * construction d'un vecteur de composantes nulles
	     */
		public Vecteur3D(){
			x = 0; y = 0; z = 0;
		}
		/**
		 * construction d'un vecteur de composantes x,y,z
		 */
		public Vecteur3D(double x,double y,double z){
			this.x = x; this.y = y; this.z = z; 
		}
		/**
		 * construction d'un vecteur de norme donne
		 * et de direction alatoire
		 * @param norme
		 * ci on utilise le fait que la projection sur un cylindre 
		 * conserve les aires.
		 */
		public Vecteur3D(double rayon){
			//System.out.println("entre dans cration vitesse");
			double phi = 2*Math.PI*rayon*Math.random();
			double h = -rayon + 2*rayon*Math.random();
			double d = Math.sqrt(rayon*rayon-h*h);
			this.x = d*Math.cos(phi);
			this.y = d*Math.sin(phi);
			this.z = h;
		}
		/**
		 * construction d'un vecteur alatoire gaussien de moyenne nulle
		 * @param sigmax cart type sur x
		 * @param sigmay cart type sur y
		 * @param sigmaz cart type sur z
		 * @param loi
		 */
		public Vecteur3D(double sigmax,double sigmay, double sigmaz, String loi){
			if(loi == "gauss"){
				this.x = sigmax*tirage.nextGaussian();
				this.y = sigmay*tirage.nextGaussian();
				this.z = sigmaz*tirage.nextGaussian();
			}
		}
	//-------------------------
		/**
		 * cration d'un nouveau vecteur par copie
		 */
		public Vecteur3D copy(){
			return new Vecteur3D(x,y,z);
		}
	//-----------------------
		public double getX(){return x;}
		public double getY(){return y;}
		public double getZ(){return z;}
	//----------------------- 
		/**
		 * remise  0 des composantes du vecteur
		 */
		public void reset(){
			x = 0; y = 0; z = 0;
		}
		/**
		 * mise  jour des composantes
		 */
		public void set(double newX, double newY, double newZ){
			x = newX; y = newY; z = newZ;
		}
		/**
		 * mise  jour  partir de nouvelles composantes polaires
		 * @param rphipsi Vecteur3D_pol des composantes polaires
		 */
		public void set(Vecteur3D_Pol rphipsi){
			double r = rphipsi.getR();
			double phi = rphipsi.getPhi();
			double psi = rphipsi.getPsi();
			x = r*Math.cos(psi)*Math.cos(phi);
			y = r*Math.cos(psi)*Math.sin(phi);
			z = r*Math.sin(psi);
		}
		/**
		 * transformation en un vecteur colinaire 
		 * @param l facteur multiplicateur
		 */
		public void line(double l){
			x = l*x; y = l*y; z = l*z;
		}
		/**
		 * combinaison linaire avec un autre vecteur
		 * @param l facteur s'appliquant au vecteur
		 * @param p facteur s'appliquant  l'autre vecteur
		 * @param vecteur autre vecteur
		 */
		public void lineWith(double l, double p, Vecteur3D vecteur){
			x = l*x+p*vecteur.getX();y = l*y+p*vecteur.getY();z = l*z+p*vecteur.getZ();
		}
		/**
		 * transformation du vecteur par produit vectoriel avec un autre vecteur
		 * @param vecteur
		 */
		public void vectWith(Vecteur3D vecteur){
			double newX = y*vecteur.getZ() - z*vecteur.getX();
			double newY = z*vecteur.getX() - x*vecteur.getY();
			double newZ = x*vecteur.getY() - y*vecteur.getX();
			x = newX; y= newY; z = newZ; 
		} 
		/**
		 * produit scalaire avec un autre vecteur
		 * @param vecteur
		 * @return produit scalaire
		 */
		public double scalWith(Vecteur3D vecteur){
			return x*vecteur.getX()+y*vecteur.getY()+z*vecteur.getZ();
		}
		/**
		 * norme du vecteur
		 * @return la norme
		 */
		public double norme(){
			return Math.sqrt(x*x+y*y+z*z);
		}
		/**
		 * transformation en coordonnes polaires 
		 * @return le vecteur polaire r, phi, psi associ
		 */
		public Vecteur3D_Pol repr_polaire(){
			double r, phi, psi;
			double sc = x*x+y*y;
			double rc = sc+z*z;
			r = Math.sqrt(rc);
			psi = Math.asin(z/r);
			if (y>=0){
				phi = Math.acos(x/rc);
			}
			else{
				phi = 2*Math.PI - Math.acos(x/rc);
			}
			return new Vecteur3D_Pol(r,phi,psi);
		}
}
