package espace3D;


public class Vecteur3D_Pol {
	/**triplets de nombres rels utilisables 
	 * pour reprsenter des vecteurs 
	 * ou des points d'un espace affine
	 * dans un repre polaire 
	 * r composante radiale 0<=r
	 * phi dans le plan Ox,Oy 0 <= phi < 2Pi
	 * psi -Pi/2 <= psi <= +Pi/2
	 */
		protected double r, phi, psi;
	/**
	 * construction d'un triplet polaire de composantes nulles
	 */
		public Vecteur3D_Pol(){
			this.r  = 0; this.phi = 0; this.psi = 0;
		}
	/**
	 * construction d'un triplet polaire de composantes r,phi,psi
	 * @param r composante radiale
	 * @param phi
	 * @param psi
	 */
		public Vecteur3D_Pol(double r, double phi, double psi){
			this.r = r; this.phi = phi; this.psi = psi;
		}
	//-------------------------------//-----------------------
		public double getR(){return r;}
		public double getPhi(){return phi;}
		public double getPsi(){return psi;}
	//----------------------- 
		public void reset(){
			r = 0; phi = 0; psi = 0;
		}
		public void set(double newR, double newPhi, double newPsi){
			r = newR; phi = newPhi; psi = newPsi;
		}
		/**
		 * mise  jour  partir de nouvelles composantes cartsiennes
		 * @param xyz Vecteur3D portant les composantes cartsiennes
		 */
		public void set(Vecteur3D xyz){
			double x = xyz.getX();double y = xyz.getY();double z = xyz.getZ();
			double sc = x*x+y*y;
			double rc = sc+z*z;
			r = Math.sqrt(rc);
			psi = Math.asin(z/r);
			if (y>=0){
				phi = Math.acos(x/rc);
			}
			else{
				phi = 2*Math.PI - Math.acos(x/rc);
			}
		}
		/**
		 * transformation en coordonnes cartsiennes
		 * @return retourne le triplet x,y,z associ
		 */
		public Vecteur3D repr_cart(){
			double x = r*Math.cos(psi)*Math.cos(phi);
			double y = r*Math.cos(psi)*Math.sin(phi);
			double z = r*Math.sin(psi);
			return new Vecteur3D(x,y,z);
		}
}
